# Beeper Go SDK

- [**CLI Mode**](#cli-mode)
- [**Building for Android**](#building-for-android)
- [**Building for iOS**](#building-for-ios)
- [**Building for Desktop**](#building-for-desktop)

## CLI Mode

This repo includes a REPL/CLI for testing out FFI calls in `./cmd/cli`. To run it you'll need to build `libsignal_ffi.a` first, roughly:

```sh
git submodule update --init --recursive
cd pkg/bridges/megasignal/signal
./build.sh

# This is macOS specific, assuming you have libolm LIBRARY_PATH already set
LIBRARY_PATH=$LIBRARY_PATH:pkg/bridges/megasignal/signal/ gow run ./cmd/cli
```

-----

There is also an interactive version of the CLI called `megacli`. The recommended way to run it is using our helper script:

```bash
# Run with the default profile (stored in ~/.megacli/default/)
./scripts/megacli.sh

# Run with a specific profile
./scripts/megacli.sh -p staging

# Or specify a relative directory for data
./scripts/megacli.sh --relative-datadir=path/to/data
```

You can also view logs and access the SQLite database with these helper scripts:

```bash
# tail -f logs for a profile (default or custom)
./scripts/megaclilogs.sh
./scripts/megaclilogs.sh -p staging

# Open the database for a profile with litecli
./scripts/megaclidb.sh
./scripts/megaclidb.sh -p staging
```

**NOTE:** Don't run cli and megacli at the same time if they share the same data directory!

-----

Once you have the CLI running you can call FFI API functions. These are available via JSON or via shortcuts when there are no arguments, these call pairs are identical:

```sh
# Start the entire SDK (Matrix client + all bridges)
{"command":"Start","data":[]}
Start

# Only start the Matrix client
{"command":"matrix.Start","data":[]}
matrix.Start
```

Note: If you're using megacli, all these commands work, but prefixed with `call` and without the JSON formatting (except for where JSON is actually needed).
Spaces need to be escaped.

For example:
```sh
call matrix.Start
call matrix.LoginWithUsername username mysecretpassword https://matrix.beeper-staging.com
call matrix.VerifyWithRecoveryCode ABCDEF
call SendEvent {"room_id":"!WjYBrkvWTFATiu_k:ba_uVe00BzyvJD6iTqf.local-signal.localhost","type":"m.room.message","content":{"body":"hi\ there","msgtype":"m.text"}} transaction_id
```


Matrix APIs:

```sh
# Login
{"command":"matrix.LoginWithUsername","data":["username", "mysecretpassword", "https://matrix.beeper-dev.com"]}

# Verify with recovery code
{"command":"matrix.VerifyWithRecoveryCode","data":["ABC DEF"]}

# Or go throigh cross signing emoji verification
{"command":"matrix.StartVerification","data":["@username:beeper-dev.com"]}
{"command":"matrix.ConfirmSAS","data":["TRANSACTION_ID_FROM_START"]}

# Send events
{"command":"SendEvent","data":["{\"room_id\":\"the-room-id\",\"type\":\"m.room.message\",\"content\":{\"body\":\"hi\",\"msgtype\":\"m.text\"}}","unique-txn-id-must-change-each-event"]}

# Logout
{"command":"matrix.Logout","data":[]}

# Get Matrix client state
{"command":"matrix.GetState","data":[]}
```

And local bridges APIs:

```sh
# (Optional) Enable running local bridges without Matrix login
{"command":"bridges.SetAnonymousUserID","data":["@username:beeper-dev.com"]}

# Start all local bridges
{"command":"bridges.Start","data":[]}
bridges.Start

# Start/stop a specific local bridge
{"command":"bridges.StartBridge","data":["local-dummy"]}
{"command":"bridges.StopBridge","data":["local-dummy"]}

# Initialize (create, don't start) local bridge
{"command":"bridges.InitBridge","data":["local-dummy"]}

# Enable dummybridge
{"command":"SetFeatureFlag","data":["dummy", true]}
```

## Building for Android

1. Make sure your submodules are looking good (`git submodule update --init --recursive`)

2. Install go and rust toolchain (rustup, cargo, etc)

3. Install gomobile (https://pkg.go.dev/golang.org/x/mobile/cmd/gomobile)
  ```
  go install golang.org/x/mobile/cmd/gomobile@latest
  gomobile init
  ```
4. Install the Android SDK from Android Studio or using the Command Line Tools: https://developer.android.com/tools

5. Using android studio or sdkmanager install "tools" "platform-tools" and "ndk;26.1.1090912"

6. Tell gradle where to find your android sdk.  Under the android/ directory add a file named `local.properties` with contents like
   ```
   sdk.dir=/Users/adamv/Library/Android/sdk
   ```
   Substitute the real path to your android sdk

7. Build the aar

  ```
  cd android
  ./gradlew -p library assemble
  ```

### Android Emulator Devloop

1. Clone `beeper-android`
2. Open in Android studio
3. Edit `build.gradle.kts` and change implementation to `./android/library/build/outputs/aar/library-release.aar`
4. (re)Build and run


## Building for iOS

1. Make sure your submodules are looking good (`git submodule update --init --recursive`)

2. Xcode tools and Rust must be installed

  ```sh
  curl --proto '=https' --tlsv1.2 -sSf https://sh.rustup.rs | sh
  ```

3. Rust targets must be added

  ```sh
  rustup target add aarch64-apple-ios aarch64-apple-ios-sim x86_64-apple-ios
  ```

4. Install Go (check [`.github/workflows/ios.yml`](./.github/workflows/ios.yml) to see what version the CI uses)

5. Install gomobile (https://pkg.go.dev/golang.org/x/mobile/cmd/gomobile)

  ```sh
  go install golang.org/x/mobile/cmd/gomobile@latest
  gomobile init
  ```

6. Run build script

  ```sh
  ./scripts/build-ios.sh
  ```


## Building for Desktop

1. Make sure your submodules are looking good (`git submodule update --init --recursive`)

2. You will need:

- [Go](https://go.dev) (check [`.github/workflows/ios.yml`](./.github/workflows/ios.yml) to see what version iOS builds use)
- [Python](https://www.python.org/)
- [Node.js](https://nodejs.org/)
- [CMake](https://cmake.org)
- Make sure [Corepack](https://nodejs.org/api/corepack.html) is enabled (run `corepack enable`). This should make `yarn` available.
  - Otherwise, [Yarn v4](https://yarnpkg.com/) can be installed manually
- For macOS, to make sure your command line tools (provides C compiler, etc.) are installed by running `xcode-select --install` if you haven't done so already

3. Download libsignal binary

```bash
./scripts/prepare-libsignal-desktop.sh
```

4. Install node dependencies

```bash
yarn install
```

5. Build

```bash
yarn build
# or if that didn't work somehow
node node/scripts/node-pre-gyp-wrapper.mjs
```

If you are developing the Node bindings and want to easily test on Desktop, command below will build the SDK and will replace the native module in `../beeper-desktop-new/node_modules/@beeper/beeper-client-sdk`. This assumes you have the desktop repository and the SDK repository in the same directory.

```bash
./scripts/build-and-install-node-module.sh
```

### Usage/Testing/REPL

[`node/beeper_client_test.js`](./node/beeper_client_test.js) is a Node.js script that consumes the [Node-API](https://nodejs.org/api/n-api.html) bindings, instantiating the SDK and logging in with an account. To run it, you need to set the following environment variables:

| Environment Variable | Notes | Required | Default |
|---------------------|-------------|-----------|----------|
| `BEEPER_USERNAME` | Your Beeper username | Required | — |
| `BEEPER_PASSWORD` | Your Beeper password | Required | — |
| `BEEPER_RECOVERY_CODE` | Your Beeper recovery code; verification will fail without it | "Optional" | — |
| `BEEPER_DATA_DIR` | Path to the data directory | Optional | `./storage` |
| `BEEPER_HOMESERVER` | Homeserver URL (for staging, use `https://matrix.beeper-staging.com` instead) | Optional | `https://matrix.beeper.com` |


You can run the script like this:

```bash
cd node

# update these to your actual credentials
export BEEPER_USERNAME=batuhan
export BEEPER_PASSWORD=very-strong-password
export BEEPER_RECOVERY_CODE="NoTm yAct UALr eCoV eRYK EYDO notw oWorr y000 0000 0000 0000"
export BEEPER_DATA_DIR=../storage # optional
export BEEPER_HOMESERVER=https://matrix.beeper.com # optional

# first run, should login and output state
# pass `--log-callbacks` to log `Write`/`Sync`, etc. (intended to be viewed from attached Chrome DevTools)
node beeper_client_test.js --log-callbacks

# to test logout after login
node beeper_client_test.js --logout
```

This spawns a primitive REPL that, for now, only lets you `quit`; however, an [inspector session](https://nodejs.org/api/inspector.html) is automatically started, which lets you attach to the process from Chrome DevTools. To do this, open any Chromium-based browser and navigate to **chrome://inspect**; you should see the Node.js process once login has completed. Access the "Console" tab to see logs and call SDK methods.

The `BeeperClient` is available through the `sdk` global. For example, to perform
a local login process for Google Chat, one might do:

```js
await sdk.start();
let bridge = "local-googlechat";
let processId = await sdk.bridges.localLoginCreateProcess(bridge, "cookies");
await sdk.bridges.localLoginStartProcess(bridge, processId);
await sdk.bridges.localLoginProcessSubmitCookies(bridge, proc, JSON.stringify({
  "COMPASS": "…",
  "SSID": "…",
  "SID": "…",
  "OSID": "…",
  "HSID": "…"
}));

// bridge should auth from here
```

(The "subobjects" (`sdk.bridges`, `sdk.matrix`) are mapped in [./node/index.js](./node/index.js).
Available methods may be queried from the console itself.)

#### Using 1Password

If you use [1Password](https://1password.com/) and have the [1Password CLI](https://developer.1password.com/docs/cli/get-started/) installed, then you can automatically pull the credentials using [secret references](https://developer.1password.com/docs/cli/secret-references/):

```bash
item="…" # "item UUID", which can be copied from a three dots menu in the 1Password desktop app
BEEPER_HOMESERVER="https://matrix.beeper-staging.com" \
      BEEPER_USERNAME="op://Personal/$item/username" \
      BEEPER_PASSWORD="op://Personal/$item/password" \
      BEEPER_RECOVERY_CODE="op://Personal/$item/recovery code" \
      op run --no-masking -- node beeper_client_test.js --log-callbacks --wait-for-debugger
```

`BEEPER_RECOVERY_CODE` is populated with a custom "recovery code" field that is
assumed to be added to the item in 1Password. This can be added by selecting
"add more" while editing an item and specfiying "Text". Rename the name of the field to
"recovery code" and fill the value accordingly.

#### Flags

| Flag | Notes|
|------|-------------|
| `--log-callbacks` | Logs incoming data received from "callback invocations" (instances of SDK calling the function passed as `callback` in the options object). The output is optimized to be viewed in an attached Chrome DevTools, and isn't appropriate for e.g. being piped to a file. |
| `--wait-for-debugger` | Halts execution before logging in, waiting for an inspector to be attached before continuing. Useful to see early logs in DevTools. This implicitly prevents the default behavior of passively accepting inspectors after logging in. |
| `--no-inspector` | Prevents the inspector listener from being automatically created after login. Doesn't suppress `--wait-for-debugger`. |
| `--logout` | Signs out the currently logged in user after logging in |
| `--list-devices` | Outputs all devices associated with the account to stdout. |
| `--dump-rageshake` | Dumps the response from `FetchDataForRageshake` to stdout. |

> [!NOTE]
> When `--log-callbacks` is used, the logs are best viewed from an attached Chrome
> DevTools. It's important to note that the logs are **not printed verbatim**, instead
> optimized for readability and skimming.
>
> Examine the `beeper_client_test.js` script for more information.

### Code Generation

Node bindings use code generation to handle boilerplate required for FFI via [Node-API](https://nodejs.org/api/n-api.html). If any of the exposed methods are added/removed/changed in the SDK, you'll need to regenerate `beeper_client_generated.go` manually.

```bash
# Only tested on macOS
yarn generate
```

### Releases

Development environments pull the latest [Release](https://github.com/beeper/beeper-client-sdk/releases).

To create a new build for release, navigate to the [Node - Create Release](https://github.com/beeper/beeper-client-sdk/actions/workflows/release-node.yml) job and run the workflow.
